module Data.SpirV.Reflect.Enums.DescriptorType where

import Data.SpirV.Reflect.Enums.Common

newtype DescriptorType = DescriptorType Int
  deriving newtype (Eq, Ord, Show, Enum)

pattern DESCRIPTOR_TYPE_SAMPLER :: DescriptorType
pattern DESCRIPTOR_TYPE_SAMPLER = DescriptorType 0

pattern DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER :: DescriptorType
pattern DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER = DescriptorType 1

pattern DESCRIPTOR_TYPE_SAMPLED_IMAGE :: DescriptorType
pattern DESCRIPTOR_TYPE_SAMPLED_IMAGE = DescriptorType 2

pattern DESCRIPTOR_TYPE_STORAGE_IMAGE :: DescriptorType
pattern DESCRIPTOR_TYPE_STORAGE_IMAGE = DescriptorType 3

pattern DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER :: DescriptorType
pattern DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER = DescriptorType 4

pattern DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER :: DescriptorType
pattern DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER = DescriptorType 5

pattern DESCRIPTOR_TYPE_UNIFORM_BUFFER :: DescriptorType
pattern DESCRIPTOR_TYPE_UNIFORM_BUFFER = DescriptorType 6

pattern DESCRIPTOR_TYPE_STORAGE_BUFFER :: DescriptorType
pattern DESCRIPTOR_TYPE_STORAGE_BUFFER = DescriptorType 7

pattern DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC :: DescriptorType
pattern DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC = DescriptorType 8

pattern DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC :: DescriptorType
pattern DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC = DescriptorType 9

pattern DESCRIPTOR_TYPE_INPUT_ATTACHMENT :: DescriptorType
pattern DESCRIPTOR_TYPE_INPUT_ATTACHMENT = DescriptorType 10

pattern DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR :: DescriptorType
pattern DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR = DescriptorType 1000150000

descriptorTypeName :: IsString label => DescriptorType -> Maybe label
descriptorTypeName = toLabel descriptorTypeNames

descriptorTypeNames :: IsString label => [(DescriptorType, label)]
descriptorTypeNames =
  [ (DESCRIPTOR_TYPE_SAMPLER, "SAMPLER")
  , (DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, "COMBINED_IMAGE_SAMPLER")
  , (DESCRIPTOR_TYPE_SAMPLED_IMAGE, "SAMPLED_IMAGE")
  , (DESCRIPTOR_TYPE_STORAGE_IMAGE, "STORAGE_IMAGE")
  , (DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER, "UNIFORM_TEXEL_BUFFER")
  , (DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER, "STORAGE_TEXEL_BUFFER")
  , (DESCRIPTOR_TYPE_UNIFORM_BUFFER, "UNIFORM_BUFFER")
  , (DESCRIPTOR_TYPE_STORAGE_BUFFER, "STORAGE_BUFFER")
  , (DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, "UNIFORM_BUFFER_DYNAMIC")
  , (DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, "STORAGE_BUFFER_DYNAMIC")
  , (DESCRIPTOR_TYPE_INPUT_ATTACHMENT, "INPUT_ATTACHMENT")
  , (DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR, "ACCELERATION_STRUCTURE_KHR")
  ]
